umask 022
barf() { echo "build-dist: fatal: $1" >&2; exit 111; }
usage() {
  echo "build-dist: usage: build-dist [ -b -m ] project [ version tmpdir ]" 1>&2
  exit 100
}

here=`env - PATH=$PATH pwd`

target=make
theversion=0
while test $# -gt 0
do
  case $1 in
    -b)
        target=build
        shift
        ;;
    -m)
        target=make
        shift
        ;;
     -*)
	usage
	;;
     *)
	break
	;;
  esac
done

if test $# -eq 0
then
  usage
fi

project=$1
version=$2
tmpdir=${3-"tmp"}
package=`basename $project`

case $project in
  /*)
    barf "project name cannot begin with /"
    ;;
  *)
    ;;
esac

if test -r package/dist
then
  dist=`sed -e '/^$/q' <package/dist`
else
  if test -r HOME/conf/dist
  then
    dist=`sed -e '/^$/q' <HOME/conf/dist`
  else
    barf "cannot find dist file"
  fi
fi

if test -r package/exclude
then
  exclude=`sed -e '/^$/q' <package/exclude || exit 0`
else
  if test -r HOME/conf/exclude
  then
    exclude=`sed -e '/^$/q' <HOME/conf/exclude || exit 0`
  else
    barf "cannot find exclude file"
  fi
fi

test -d $tmpdir && barf "$tmpdir/ already exists"

trap "cd $here && rm -fr $tmpdir" 0

if test "X$version" != "X"
then
  project="$project-$version"
fi

mkdir -p $tmpdir/$project
chmod -R 01755 $tmpdir/*
chmod 0755 $tmpdir/$project

TARC $dist | ( cd $tmpdir/$project && TARX )

lsdir() {
  for i in `ls $1`
  do
    if test -d $1/$i
    then
      lsdir $1/$i
    else
      echo $1/$i
    fi
  done
}   

( cd $tmpdir/$project
  for i in `ls`
  do 
    if test -d $i
    then
      echo $i
      lsdir $i
    else
      echo $i
    fi
  done
) > $tmpdir/files.tmp

if test "X$version" != "X"
then
  if test -d $tmpdir/$project/package
  then
    for f in `grep -l VERSION $tmpdir/$project/package/* || exit 0`
    do
      sed "s}VERSION}$version}g" <$f >$f.tmp
      cat $f.tmp >$f
      rm -f $f.tmp
    done
    for f in `grep -l PACKAGE $tmpdir/$project/package/* || exit 0`
    do
      sed "s}PACKAGE}$package}g" <$f >$f.tmp
      cat $f.tmp >$f
      rm -f $f.tmp
    done
  fi
fi

( for f in $exclude
  do
    rm -fr $tmpdir/$project/$f
  done

  if test "$target" = "build"
  then
    for f in `cat $tmpdir/files.tmp`
    do
      if test -r $tmpdir/$project/$f
      then
	echo $f
      fi
    done
  else
    for f in `cat $tmpdir/files.tmp`
    do
      case $f in
	*.do)
	  rm -f $tmpdir/$project/$f
	  ;;
	*=*)
	  rm -f $tmpdir/$project/$f
	  ;;
	*)
	  if test -r $tmpdir/$project/$f
	  then
	    echo $f
	  fi
	  ;;
      esac
    done
  fi
) > $tmpdir/$project/package/files
rm -f $tmpdir/files.tmp

( cd $tmpdir/$project && touch `cat package/files` )

( cd $tmpdir && TARC * ) | ZIP

exit 0

