umask 022
build_barf() { echo "build-target: fatal: $@" 1>&2; exit 100; }
build_error() { echo "$@" 1>&2; exit 111; }
build_say() { test $build_verbose -eq 0 || echo "$@" 1>&2; }
build_trace() { test $build_verbose -lt 2 || echo "$@" 1>&2; }
build_here=`env - PATH=$PATH pwd`
build_borrow=-1
build_verbose=1
build_usage() {
  echo "build-target: usage: build-target [ -bBfFvVq- ] target" >&2
  exit 100;
}

build_self=$0
while test $# -gt 0
do
  case "$1" in
    -b) build_borrow=1; build_self="$build_self $1"; shift ;;
    -B) build_borrow=0; build_self="$build_self $1"; shift ;;
    -q) build_verbose=0; build_self="$build_self $1"; shift ;;
    -v) build_verbose=1; build_self="$build_self $1"; shift ;;
    -V) build_verbose=2; build_self="$build_self $1"; shift ;;
    --) shift; break ;;
    -*) build_usage ;;
     *) break ;;
  esac
done

depend() {
  :
}

# formake line
formake() {
  build_trace formake: $@
  if test $build_update -eq 0
  then
    rm -f $build_target.tmp make/$build_target.tmp
    exit 0
  fi
  build_say "$@"
  if test $# -eq 0
  then
    return 0
  fi
  echo "	$@" >> make/$build_target.tmp \
    || barf "cannot write Makefile"

  return 0
}

build_update_it() {
  build_trace build_update_it: $@
  if test -r $1
  then
    if test -r make/$1
    then
      HOME/command/build-update make/$1 make/Makefile.tmp || return 0
    fi
  fi
  $build_self $1 || return 1
  return 0
}

build_uptodate() {
  HOME/command/build-update $@ || return 0
  build_update=1
  rm -f make/$1
  return 0
}

# dependon dep ...
dependon() {
  build_trace dependon: $@
  if test $# -eq 0
  then
    return 0
  fi
  if test $build_update -eq -1
  then
    build_update=0
  fi
  for i in $@
  do
    build_update_it $i || exit 1
  done
  build_uptodate $build_target $@
  for i in $@
  do
    case $i in
      *=*|*.do) :
	;;
      *)
	echo $i >> make/$build_target.tmp \
	  || build_failed "cannot update Makefile"
	;;
    esac
  done
  return 0
}

# dependcc target
dependcc() {
  build_trace dependcc: $@
  if test $# -eq 0
  then
    return 0
  fi
  dependon $@ || exit 1
  build_cpp=`awk '
      BEGIN { FS="\""; ORS=" " }
      /^[ \t]*#[ \t]*include[ \t][ \t]*"[^"]*"[ \t]*$/ {
        print $2
      }
    ' $@`
  dependcc $build_cpp
}

# directtarget
directtarget() {
  build_trace directtarget: $build_target
  rm -f $build_target.tmp
}

# nosuchtarget
nosuchtarget() {
  build_trace nosuchtarget: $build_target $build_ext
  if test $build_borrow -eq 0
  then
    HOME/command/build-nosuchtarget -B -- $build_target $build_ext \
      || exit $?
  else
    HOME/command/build-nosuchtarget -b -- $build_target $build_ext \
      || exit $?
  fi
  exit 0
}

build_newtarget() {
  build_trace build_newtarget: $@
  touch make/$1.tmp || build_failed "cannot update Makefile"
  mv make/$1.tmp make/$1 || build_failed "cannot update Makefile"
  return 0
}

build_fail() {
  build_trace build_fail: $@
  rm -f make/$1 make/$1.tmp $1.tmp
  build_error "cannot build: $1"
}

build_failed() {
  build_trace build_failed: $@
  rm -f make/$build_target make/$build_target.tmp $build_target.tmp
  build_error "cannot build: $build_target: $@"
}

build_doit() {
  build_trace build_doit: $build_dofile $@
  ( set -e; . ./$build_dofile $1 $2 $3; exit 0 )
}

# build_finddo $target $ext
build_finddo() {
  build_trace build_finddo: $@
  build_dofile=`basename $1.do`
  if test -r $build_dofile
  then
    return 0
  fi
  build_dofile=default$2.do
  if test -r default$2.do
  then
    return 0
  fi
  return 1
}

# build_dotit dofile target ext tmp
build_dotit() {
  build_trace build_dotit: $@
  build_err=99
  while test $build_err -eq 99
  do
    build_target=$1
    build_ext=$2
    case $1 in
      */*)
	build_base=`dirname $1`/`basename $1 $2`
	;;
      *)
	build_base=`basename $1 $2`
	;;
    esac
    if build_finddo $1 $2
    then
      build_update=-1
      build_doit $1 $build_base $1.tmp 1>$build_target.tmp
      build_err=$?
    else
      build_trace build-nosuchtarget: $build_base $build_ext
      HOME/command/build-nosuchtarget $build_base $build_ext 
      build_err=$?
    fi
  done

  if test $build_err -eq 100
  then
    rm -f make/$1 make/$1.tmp $1.tmp
    return 0
  fi

  if test $build_err -eq 111
  then
    build_failed "no such target"
  fi

  if test $build_err -eq 0
  then
    if test -r src/$1
    then
      return 0
    fi
    if test -r $1.tmp
    then
      touch $1.tmp || build_fail $1
      mv $1.tmp $1 || build_fail $1 
    fi
    if test -r make/$1.tmp
    then
      build_newtarget $1
      return 0
    fi
    test -r make/$1 || build_failed "no make file"
    touch make/$1 || build_failed "cannot update Makefile"
    return 0
  fi

  build_fail $1
}

# build_rebuild target ext
build_rebuild() {
  build_trace build_rebuild: $@
  if test -r $1
  then
    if test -r make/$1
    then
      HOME/command/build-update make/Makefile.tmp make/$1 \
        && return 0
    fi
  fi
  test -d make/`dirname $1` || mkdir -p make/`dirname $1`
  build_dotit "$1" "$2" "$1.tmp" || exit 1
  return 0
}

build_it() {
  build_trace build_it: $@
  for i in $@
  do
    if test -r $i
    then
      test -r make/$i || continue
    fi
    case $i in
      *=*|*.do) :
	;;
      *.*)
	ext=`echo $i | sed 's/.*\.\([^.]*\)$/\1/'`
	build_rebuild $i .$ext
	;;
      *)
	build_rebuild $i ''
	;;
    esac
  done
}

####
build_trace build-target: $@
build_it $@ 1>&2
