umask 022
build_barf() { echo "build: fatal: $@" 1>&2; exit 100; }
build_error() { echo "$@" 1>&2; exit 111; }
build_say() { test $build_verbose -eq 0 || echo "$@" 1>&2; }

test -d src || build_barf "no src directory"

build_usage() {
  echo "build: usage: build [ -bBfFvVq- ] [ target ]" >&2
  exit 100;
}

####
build_force=0
build_target=HOME/command/build-target
while test $# -ne 0
do
  case "$1" in
    -f) build_force=1; shift ;;
    -F) build_force=2; shift ;;
    -b|-B|-f|-F|-q|-v|-V)
      build_target="$build_target $1"
      shift ;;
    --) shift; break ;;
    -*) build_usage ;;
     *) break ;;
  esac
done

if test $build_force -eq 1
then
  for i in $@
  do
    rm -fr compile/$i
  done
else
  if test $build_force -eq 2
  then
    rm -fr make compile
  fi
fi

build_here=`env - PATH=$PATH pwd`
mkdir -p compile make command
test -r compile/home || echo $build_here > compile/home
test -h compile/src || ln -s $build_here/src compile/src
test -h compile/make || ln -s $build_here/make compile/make
link_src() {
  test $# -ne 3 && return 1
  if test -d src/$1$2
  then
    mkdir -p compile/$1$2
    for i in `ls src/$1$2`
    do
      link_src $1$2/ $i "../$3"
    done
  else
    test -h compile/$1$2 || ln -s $3/$1$2 compile/$1$2
  fi
}
for i in `ls src`
do
  link_src '' $i 'src'
done
touch $build_here/make/Makefile.tmp
sleep 1
cd compile
if test $# -eq 0
then
  $build_target it
else
  $build_target $@
fi
rm -f $build_here/make/*.tmp
{
  cat <<EOF
# Don't edit Makefile!  Use conf-* for configuration.

SHELL=/bin/sh

default: it

EOF

  {
    build_clean=''
    for i in `ls $build_here/make`
    do
      test -s $build_here/make/$i && {
	build_clean="$build_clean $i"
      }
    done
    echo clean:
    echo '	rm -f' $build_clean
   
    for i in $build_clean
    do
      echo ""
      echo "$i:"
      cat $build_here/make/$i
    done
  } | HOME/command/build-makefile
} > Makefile.tmp \
  && mv Makefile.tmp src/Makefile \
  || build_error "cannot update Makefile"

